# coding=utf-8
"""
Forum views
"""
from __future__ import absolute_import, print_function

from datetime import date, datetime
from itertools import groupby
from urllib import quote

import sqlalchemy as sa
from flask import (current_app, flash, g, make_response, render_template,
                   request)
from flask_babel import format_date
from flask_login import current_user
from sqlalchemy.orm import joinedload
from werkzeug.exceptions import BadRequest, NotFound

from abilian.core.util import utc_dt
from abilian.i18n import _, _l
from abilian.sbe.apps.communities.blueprint import Blueprint
from abilian.sbe.apps.communities.views import default_view_kw
from abilian.web import url_for, views
from abilian.web.action import ButtonAction, Endpoint
from abilian.web.nav import BreadcrumbItem
from abilian.web.views import default_view

from .forms import PostEditForm, PostForm, ThreadForm
from .models import Post, PostAttachment, Thread
from .tasks import send_post_by_email

# TODO: move to config
MAX_THREADS = 30

forum = Blueprint("forum",
                  __name__,
                  url_prefix="/forum",
                  template_folder="templates")
route = forum.route


def post_kw_view_func(kw, obj, obj_type, obj_id, **kwargs):
    """
    kwargs for Post default view
    """
    kw = default_view_kw(kw, obj.thread, obj_type, obj_id, **kwargs)
    kw['thread_id'] = obj.thread_id
    kw['_anchor'] = u'post_{:d}'.format(obj.id)
    return kw


@forum.url_value_preprocessor
def init_forum_values(endpoint, values):
    g.current_tab = 'forum'

    g.breadcrumb.append(BreadcrumbItem(
        label=_l(u'Conversations'),
        url=Endpoint('forum.index', community_id=g.community.slug)))


@route('/')
def index():
    query = Thread.query \
        .filter(Thread.community_id == g.community.id) \
        .order_by(Thread.created_at.desc())
    has_more = query.count() > MAX_THREADS
    threads = query.limit(MAX_THREADS).all()
    return render_template("forum/index.html",
                           threads=threads,
                           has_more=has_more)


def group_monthly(entities_list):
    # We're using Python's groupby instead of SA's group_by here
    # because it's easier to support both SQLite and Postgres this way.
    def grouper(entity):
        return entity.created_at.year, entity.created_at.month

    def format_month(year, month):
        month = format_date(date(year, month, 1), "MMMM").capitalize()
        return u"%s %s" % (month, year)

    grouped_entities = groupby(entities_list, grouper)
    grouped_entities = [(format_month(year, month), list(entities))
                        for (year, month), entities in grouped_entities]
    return grouped_entities


@route('/archives/')
def archives():
    all_threads = Thread.query \
        .filter(Thread.community_id == g.community.id) \
        .order_by(Thread.created_at.desc()).all()

    grouped_threads = group_monthly(all_threads)
    return render_template('forum/archives.html',
                           grouped_threads=grouped_threads)


@route('/attachments/')
def attachments():
    all_threads = Thread.query \
        .filter(Thread.community_id == g.community.id) \
        .options(joinedload('posts')) \
        .options(joinedload('posts.attachments')) \
        .order_by(Thread.created_at.desc()).all()

    posts_with_attachments = []
    for thread in all_threads:
        for post in thread.posts:
            if getattr(post, 'attachments', None):
                posts_with_attachments.append(post)
    posts_with_attachments.sort(key=lambda post: post.created_at)
    posts_with_attachments.reverse()

    grouped_posts = group_monthly(posts_with_attachments)
    return render_template('forum/attachments.html',
                           grouped_posts=grouped_posts)


class BaseThreadView(object):
    Model = Thread
    Form = ThreadForm
    pk = 'thread_id'
    base_template = 'community/_base.html'

    def can_send_by_mail(self):
        return (g.community.type == 'participative' or
                g.community.has_permission(current_user, 'manage'))

    def prepare_args(self, args, kwargs):
        args, kwargs = super(BaseThreadView, self).prepare_args(args, kwargs)
        self.send_by_email = False

        if not self.can_send_by_mail() and 'send_by_email' in self.form:
            # remove from html form and avoid validation errors
            del self.form['send_by_email']

        return args, kwargs

    def index_url(self):
        return url_for(".index", community_id=g.community.slug)

    def view_url(self):
        return url_for(self.obj)


class ThreadView(BaseThreadView, views.ObjectView):
    methods = ['GET', 'HEAD']
    Form = PostForm
    template = 'forum/thread.html'

    @property
    def template_kwargs(self):
        kw = super(ThreadView, self).template_kwargs
        kw['thread'] = self.obj
        kw['is_closed'] = self.obj.closed
        return kw


thread_view = ThreadView.as_view('thread')
default_view(forum, Post, None, kw_func=post_kw_view_func)(thread_view)
default_view(forum, Thread, 'thread_id', kw_func=default_view_kw)(thread_view)

route('/<int:thread_id>/')(thread_view)
route('/<int:thread_id>/attachments')(ThreadView.as_view(
    'thread_attachments',
    template='forum/thread_attachments.html'))


class ThreadCreate(BaseThreadView, views.ObjectCreate):
    POST_BUTTON = ButtonAction('form',
                               'create',
                               btn_class='primary',
                               title=_l(u'Post this message'))

    title = _("New conversation")

    def init_object(self, args, kwargs):
        args, kwargs = super(ThreadCreate, self).init_object(args, kwargs)
        self.thread = self.obj
        return args, kwargs

    def before_populate_obj(self):
        del self.form['attachments']
        self.message_body = self.form.message.data
        del self.form['message']

        if 'send_by_email' in self.form:
            self.send_by_email = (self.can_send_by_mail() and
                                  self.form.send_by_email.data)
            del self.form['send_by_email']

    def after_populate_obj(self):
        if self.thread.community is None:
            self.thread.community = g.community._model

        self.post = self.thread.create_post(body_html=self.message_body)
        obj_meta = self.post.meta.setdefault('abilian.sbe.forum', {})
        obj_meta['origin'] = u'web'
        obj_meta['send_by_email'] = self.send_by_email

        session = sa.orm.object_session(self.thread)
        uploads = current_app.extensions['uploads']

        for handle in request.form.getlist('attachments'):
            fileobj = uploads.get_file(current_user, handle)
            if fileobj is None:
                continue

            meta = uploads.get_metadata(current_user, handle)
            name = meta.get('filename', handle)
            mimetype = meta.get('mimetype')

            if not isinstance(name, unicode):
                name = unicode(name, encoding='utf-8', errors='ignore')

            if not name:
                continue

            attachment = PostAttachment(name=name)
            attachment.post = self.post

            with fileobj.open('rb') as f:
                attachment.set_content(f.read(), mimetype)
            session.add(attachment)

    def commit_success(self):
        if self.send_by_email:
            task = send_post_by_email.delay(self.post.id)
            meta = self.post.meta.setdefault('abilian.sbe.forum', {})
            meta['send_post_by_email_task'] = task.id
            self.post.meta.changed()
            session = sa.orm.object_session(self.post)
            session.commit()

    @property
    def activity_target(self):
        return self.thread.community

    def get_form_buttons(self, *args, **kwargs):
        return [self.POST_BUTTON, views.object.CANCEL_BUTTON]


route('/new_thread/')(ThreadCreate.as_view('new_thread',
                                           view_endpoint='.thread'))


class ThreadPostCreate(ThreadCreate):
    """Add a new post to a thread."""
    methods = ['POST']
    Form = PostForm
    Model = Post

    def init_object(self, args, kwargs):
        # we DO want to skip ThreadCreate.init_object. hence super is not based on
        # ThreadPostCreate
        args, kwargs = super(ThreadPostCreate, self).init_object(args, kwargs)
        thread_id = kwargs.pop(self.pk, None)
        self.thread = Thread.query.get(thread_id)
        return args, kwargs

    def after_populate_obj(self):
        super(ThreadPostCreate, self).after_populate_obj()
        session = sa.orm.object_session(self.obj)
        session.expunge(self.obj)
        self.obj = self.post


route('/<int:thread_id>/')(ThreadPostCreate.as_view('thread_post',
                                                    view_endpoint='.thread'))


class ThreadDelete(BaseThreadView, views.ObjectDelete):
    methods = ['POST']
    _message_success = _(u'Thread "{title}" deleted.')

    def message_success(self):
        return unicode(self._message_success).format(title=self.obj.title)


route('/<int:thread_id>/delete')(ThreadDelete.as_view('thread_delete'))


class ThreadCloseView(BaseThreadView, views.object.BaseObjectView):
    """Close / Re-open a thread."""
    methods = ['POST']
    _VALID_ACTIONS = {u'close', u'reopen'}
    CLOSED_MSG = _l(u'The thread is now closed for edition and new '
                    u'contributions.')
    REOPENED_MSG = _l(u'The thread is now re-opened for edition and new '
                      u'contributions.')

    def prepare_args(self, args, kwargs):
        args, kwargs = super(ThreadCloseView, self).prepare_args(args, kwargs)
        action = kwargs['action'] = request.form.get('action')

        if action not in self._VALID_ACTIONS:
            raise BadRequest(u'Unknown action: {!r}'.format(action))

        return args, kwargs

    def post(self, action=None):
        is_closed = (action == u'close')
        self.obj.closed = is_closed
        sa.orm.object_session(self.obj).commit()

        msg = self.CLOSED_MSG if is_closed else self.REOPENED_MSG
        flash(unicode(msg))
        return self.redirect(url_for(self.obj))


route('/<int:thread_id>/close')(ThreadCloseView.as_view('thread_close'))


class ThreadPostEdit(BaseThreadView, views.ObjectEdit):
    Form = PostEditForm
    Model = Post
    pk = 'object_id'

    def can_send_by_mail(self):
        # post edit: don't notify every time
        return False

    def init_object(self, args, kwargs):
        # we DO want to skip ThreadCreate.init_object. hence super is not based on
        # ThreadPostCreate
        args, kwargs = super(ThreadPostEdit, self).init_object(args, kwargs)
        thread_id = kwargs.pop('thread_id', None)
        self.thread = self.obj.thread
        assert thread_id == self.thread.id
        return args, kwargs

    def get_form_kwargs(self):
        kwargs = super(ThreadPostEdit, self).get_form_kwargs()
        kwargs['message'] = self.obj.body_html
        return kwargs

    def before_populate_obj(self):
        self.message_body = self.form.message.data
        del self.form['message']
        self.reason = self.form.reason.data

        self.send_by_email = False
        if 'send_by_email' in self.form:
            del self.form['send_by_email']

        self.attachments_to_remove = self.form['attachments'].delete_files_index
        del self.form['attachments']

    def after_populate_obj(self):
        session = sa.orm.object_session(self.obj)
        uploads = current_app.extensions['uploads']
        self.obj.body_html = self.message_body
        obj_meta = self.obj.meta.setdefault('abilian.sbe.forum', {})
        history = obj_meta.setdefault('history', [])
        history.append(dict(user_id=current_user.id,
                            user=unicode(current_user),
                            date=utc_dt(datetime.utcnow()).isoformat(),
                            reason=self.form.reason.data,))
        self.obj.meta['abilian.sbe.forum'] = obj_meta  # trigger change for SA

        attachments_to_remove = []
        for idx in self.attachments_to_remove:
            try:
                idx = int(idx)
            except ValueError:
                continue

            if idx > len(self.obj.attachments):
                continue

            attachments_to_remove.append(self.obj.attachments[idx])

        for att in attachments_to_remove:
            session.delete(att)

        for handle in request.form.getlist('attachments'):
            fileobj = uploads.get_file(current_user, handle)
            if fileobj is None:
                continue

            meta = uploads.get_metadata(current_user, handle)
            name = meta.get('filename', handle)
            mimetype = meta.get('mimetype')

            if not isinstance(name, unicode):
                name = unicode(name, encoding='utf-8', errors='ignore')

            if not name:
                continue

            attachment = PostAttachment(name=name, post=self.obj)

            with fileobj.open('rb') as f:
                attachment.set_content(f.read(), mimetype)
            session.add(attachment)


route('/<int:thread_id>/<int:object_id>/edit')(ThreadPostEdit.as_view(
    'post_edit'))


def attachment_kw_view_func(kw, obj, obj_type, obj_id, **kwargs):
    post = obj.post
    kw = default_view_kw(kw, post.thread, obj_type, obj_id, **kwargs)
    kw['thread_id'] = post.thread_id
    kw['post_id'] = post.id
    return kw


@route('/<int:thread_id>/posts/<int:post_id>/attachment/<int:attachment_id>')
@default_view(forum,
              PostAttachment,
              'attachment_id',
              kw_func=attachment_kw_view_func)
def attachment_download(thread_id, post_id, attachment_id):
    thread = Thread.query.get(thread_id)
    post = Post.query.get(post_id)
    attachment = PostAttachment.query.get(attachment_id)

    if (not (thread and post and attachment) or post.thread is not thread or
            attachment.post is not post):
        raise NotFound()

    response = make_response(attachment.content)
    response.headers['content-length'] = attachment.content_length
    response.headers['content-type'] = attachment.content_type
    content_disposition = (
        'attachment;filename="{}"'.format(quote(attachment.name.encode('utf8')))
    )
    response.headers['content-disposition'] = content_disposition
    return response



#!./bin/python
# coding=utf-8
"""
"""
from __future__ import absolute_import

import logging
from flask_script import Manager
from abilian.core.commands import setup_abilian_commands
from abilian.sbe.app import Application


def create_app():
    app = Application(name='sbe-demo')
    return app


if __name__ == '__main__':
    logging.basicConfig()
    manager = Manager(create_app)
    setup_abilian_commands(manager)

    manager.run()



#!/usr/bin/env python
# coding=utf-8
"""
"""

from __future__ import absolute_import
import logging

from flask_script import Manager
from abilian.core.commands import setup_abilian_commands
from abilian.sbe.app import Application


def create_app():
  return Application(name='myapp')


if __name__ == '__main__':
  logging.basicConfig()
  logging.getLogger().setLevel(logging.INFO)
  manager = Manager(create_app)
  setup_abilian_commands(manager)

  manager.run()




"""
Default ("home") page for social apps.
"""


def register_plugin(app):
    from .views.social import social
    from .views import users, groups, sidebars

    app.register_blueprint(social)

    # TODO: better config variable choice?
    if app.config.get("SOCIAL_REST_API"):
        from .restapi import restapi
        app.register_blueprint(restapi)



from flask_babel import lazy_gettext as _l

from abilian.core.util import BasePresenter


class CommunityPresenter(BasePresenter):

    @property
    def breadcrumbs(self):
        return [dict(label=_l("Communities"),
                     path="/communities/"), dict(label=self._model.name)]



# -*- coding: utf-8 -*-
"""
Document management blueprint.
"""
from __future__ import absolute_import

from flask import g

from abilian.i18n import _l
from abilian.sbe.apps.communities.blueprint import Blueprint
from abilian.web.action import Endpoint
from abilian.web.nav import BreadcrumbItem

from ..actions import register_actions

__all__ = ['blueprint']

blueprint = Blueprint("documents",
                      __name__,
                      url_prefix="/docs",
                      template_folder="../templates")
route = blueprint.route
blueprint.record_once(register_actions)


@blueprint.url_value_preprocessor
def init_document_values(endpoint, values):
    g.current_tab = 'documents'

    g.breadcrumb.append(BreadcrumbItem(label=_l(u'Documents'),
                                       url=Endpoint(
                                           'documents.index',
                                           community_id=g.community.slug)))



# coding=utf-8

from __future__ import unicode_literals

from flask import current_app, g, url_for
from flask_login import current_user

from abilian.i18n import _l
from abilian.services.security import Admin
from abilian.web.action import Action, actions


class CalendarAction(Action):

    def url(self, context=None):
        if self._url or self.endpoint:
            return super(CalendarAction, self).url(context=context)

        return url_for("." + self.name, community_id=g.community.slug)


def is_admin(context):
    svc = current_app.services['security']
    return svc.has_role(current_user, Admin, object=context.get('object'))

# class ForumModalAction(ModalActionMixin, ThreadAction):
#     pass
#
#
# _close_template_action = u'''
# <form method="POST" action="{{ url }}" encoding="multipart/form-data">
#   {{ csrf.field() }}
#   <button type="submit" class="btn btn-link" name="action"
#           value="{{ action.name}}">
#     {%- if action.icon %}{{ action.icon }} {% endif %}
#     {{ action.title }}
#   </button>
# </form>
# '''

_actions = [
    CalendarAction('calendar:global',
                   'new_event',
                   _l('Create a new event'),
                   icon='plus'),
    CalendarAction('calendar:global',
                   'index',
                   _l('Upcoming events'),
                   icon='list'),

    # CalendarAction('calendar:global',
    #                'calendar_view',
    #                _l('Calendar view'),
    #                icon='calendar'),
]


def register_actions(state):
    if not actions.installed(state.app):
        return
    with state.app.app_context():
        actions.register(*_actions)



"""
Some functions to retrieve activity entries.

"""
# TODO: move to the activity service ?

import sqlalchemy as sa
from flask import current_app, g
from flask_login import current_user
from werkzeug.exceptions import Forbidden

from abilian.core.extensions import db
from abilian.sbe.apps.communities.models import Membership
from abilian.sbe.apps.documents.models import Document, Folder
from abilian.services.activity import ActivityEntry
from abilian.services.security import READ, Admin


def get_recent_entries(num=20, user=None, community=None):
    AE = ActivityEntry

    # Check just in case
    if not current_user.has_role(Admin):
        if community and not community.has_member(current_user):
            raise Forbidden()

    query = AE.query.options(sa.orm.joinedload(AE.object))

    if community:
        query = query.filter(sa.or_(AE.target == g.community, AE.object ==
                                    g.community))
    if user:
        query = query.filter(AE.actor == user)

    # Security check
    #
    # we use communities ids instead of object because as of sqlalchemy 0.8 the
    # 'in_' operator cannot be used with relationships, only foreign keys values
    if not community and not current_user.has_role(Admin):
        M = Membership
        community_ids = M.query \
            .filter(M.user_id == current_user.id) \
            .values(M.community_id)

        # convert generator to list: we'll need it twice during query filtering
        community_ids = list(community_ids)
        if not community_ids:
            return []

        query = query.filter(sa.or_(
            AE.target_id.in_(community_ids), AE.object_id.in_(community_ids)))

    query = query.order_by(AE.happened_at.desc()).limit(1000)
    # get twice entries as needed, but ceil to 100
    limit = min(num * 2, 100)
    entries = []
    deleted = False
    security = current_app.services['security']
    has_permission = security.has_permission

    for entry in query.yield_per(limit):
        if len(entries) >= num:
            break

        # Remove entries corresponding to deleted objects
        if entry.object is None:
            db.session.delete(entry)
            deleted = True
            continue

        if (isinstance(entry.object, (Folder, Document)) and
                not has_permission(current_user,
                                   READ,
                                   obj=entry.object,
                                   inherit=True)):
            continue

        entries.append(entry)

    if deleted:
        db.session.commit()

    return entries



# coding=utf-8
"""
Forms for the Wiki module.
"""
from __future__ import absolute_import

from flask import g
from wtforms import HiddenField, StringField, TextAreaField
from wtforms.validators import ValidationError, required

from abilian.i18n import _, _l
from abilian.web.forms import Form
from abilian.web.forms.filters import strip
from abilian.web.forms.validators import flaghidden
from abilian.web.forms.widgets import TextArea

from .models import WikiPage


def clean_up(src):
    """Form filter."""
    src = src.replace("\r", "")
    return src


def int_or_none(val):
    try:
        return int(val)
    except (TypeError, ValueError):
        return None


class WikiPageForm(Form):
    title = StringField(label=_l(u"Title"),
                        filters=(strip,),
                        validators=[required()])
    body_src = TextAreaField(
        label=_l("Body"),
        filters=(strip, clean_up),
        validators=[required()],
        widget=TextArea(rows=10, resizeable='vertical'),)

    message = StringField(label=_l("Commit message"))
    page_id = HiddenField(filters=(int_or_none,), validators=[flaghidden()])
    last_revision_id = HiddenField(filters=(int_or_none,),
                                   validators=[flaghidden()])

    def validate_title(self, field):
        title = field.data
        if title != field.object_data and page_exists(title):
            raise ValidationError(
                _(u"A page with this name already exists. Please use another name.")
            )

    def validate_last_revision_id(self, field):
        val = field.data
        current = field.object_data

        if val is None or current is None:
            return

        if val != current:
            raise ValidationError(_(u'this page has been edited since'))


def page_exists(title):
    title = title.strip()
    return WikiPage.query \
               .filter(WikiPage.community_id == g.community.id,
                       WikiPage.title == title) \
               .count() > 0


# Not used yet
class CommentForm(Form):
    message = TextAreaField(label=_l("Message"), validators=[required()])
    page_id = HiddenField()



# coding=utf-8
"""
"""
from __future__ import absolute_import

import unittest
from cStringIO import StringIO
from itertools import count
from os.path import dirname, join
from zipfile import ZipFile

from flask import g, get_flashed_messages
from werkzeug.datastructures import FileStorage

from abilian.sbe.apps.communities.models import WRITER
from abilian.sbe.apps.communities.presenters import CommunityPresenter
from abilian.sbe.apps.communities.tests.base import (CommunityBaseTestCase,
                                                     CommunityIndexingTestCase)
from abilian.sbe.apps.documents.models import (Document, Folder,
                                               PathAndSecurityIndexable, db)
from abilian.sbe.apps.documents.views import util as view_util
from abilian.web.util import url_for


class BaseTests(CommunityBaseTestCase):
    init_data = True
    no_login = True

    def setUp(self):
        super(BaseTests, self).setUp()

        self.root_folder = Folder(title=u"root")
        db.session.add(self.root_folder)
        db.session.flush()

    @staticmethod
    def uid_from_url(url):
        return int(url.split("/")[-1])

    @staticmethod
    def open_file(filename):
        path = join(dirname(__file__), "data", "dummy_files", filename)
        return open(path, 'rb')


class TestBlobs(BaseTests):

    def test_document(self):
        root = Folder(title=u"root")
        doc = Document(parent=root, title=u"test")
        data = self.open_file("onepage.pdf").read()
        doc.set_content(data, "application/pdf")
        self.session.add(doc)
        self.session.commit()

        # coverage
        self.app.config['ANTIVIRUS_CHECK_REQUIRED'] = True
        doc.ensure_antivirus_scheduled()

    def test_antivirus_properties(self):
        root = Folder(title=u"root")
        doc = Document(parent=root, title=u"test")
        doc.set_content('content', 'text/plain')
        appcfg = self.app.config
        meta = doc.content_blob.meta

        # 1: not check required ################
        # no data in meta
        appcfg['ANTIVIRUS_CHECK_REQUIRED'] = False
        assert doc.antivirus_scanned is False
        assert doc.antivirus_status is None
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # antivirus was run, but no result
        meta['antivirus'] = None
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is None
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # virus detected
        meta['antivirus'] = False
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is False
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is False

        # virus free
        meta['antivirus'] = True
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is True
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # 2: check required ##################
        # no data in meta
        del meta['antivirus']
        appcfg['ANTIVIRUS_CHECK_REQUIRED'] = True
        assert doc.antivirus_scanned is False
        assert doc.antivirus_status is None
        assert doc.antivirus_required is True
        assert doc.antivirus_ok is False

        # antivirus was run, but no result
        meta['antivirus'] = None
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is None
        assert doc.antivirus_required is True
        assert doc.antivirus_ok is False

        # virus detected
        meta['antivirus'] = False
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is False
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is False

        # virus free
        meta['antivirus'] = True
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is True
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True


class IndexingTestCase(CommunityIndexingTestCase):

    def test_folder_indexed(self):
        folder = Folder(title=u'Folder 1', parent=self.community.folder)
        self.session.add(folder)
        folder_other = Folder(title=u'Folder 2: other', parent=self.c2.folder)
        self.session.add(folder_other)
        self.session.commit()

        svc = self.svc
        obj_types = (Folder.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search(u'folder', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search(u'folder', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == folder.object_key
            assert hit['community_slug'] == self.community.slug

        with self.login(self.user_c2):
            res = svc.search(u'folder', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == folder_other.object_key
            assert hit['community_slug'] == self.c2.slug


class TestViews(CommunityIndexingTestCase, BaseTests):

    def setUp(self):
        super(TestViews, self).setUp()
        self.community.type = 'participative'
        self.community.set_membership(self.user, WRITER)
        self.session.commit()
        self.folder = self.community.folder

    def test_util_create(self):
        with self.login(self.user):
            g.user = self.user
            g.community = CommunityPresenter(self.community)
            name = u'document'
            fs = FileStorage(
                StringIO(u'content'),
                filename=name,
                content_type='text/plain')
            doc = view_util.create_document(self.folder, fs)
            self.session.flush()
            assert doc.parent == self.folder
            assert doc.name == name

            # test upload with same name: should be renamed
            fs = FileStorage(
                StringIO(u'content'),
                filename=name,
                content_type='text/plain')
            doc2 = view_util.create_document(self.folder, fs)
            self.session.flush()
            assert doc2.parent == self.folder
            assert len(self.folder.children) == 2
            assert doc2.name == name + u'-1'
            messages = get_flashed_messages()
            assert len(messages) == 1

    def test_home(self):
        with self.client_login(self.user.email, password='azerty'):
            response = self.get(url_for('documents.index',
                                        community_id=self.community.slug))
            self.assert_status(response, 302)
            self.assertEqual(response.headers['Location'],
                             u'http://localhost/communities/{}/docs/folder/{}'
                             u''.format(self.community.slug, self.folder.id),)

    def _test_upload(self,
                     title,
                     content_type,
                     test_preview=True,
                     assert_preview_available=True):
        data = {
            'file': (self.open_file(title), title, content_type),
            'action': 'upload',
        }

        folder = self.community.folder
        url = url_for("documents.folder_post",
                      community_id=self.community.slug,
                      folder_id=folder.id)
        response = self.client.post(url, data=data)
        self.assert_status(response, 302)

        doc = folder.children[0]
        assert doc.title == title
        url = url_for("documents.document_view",
                      community_id=self.community.slug,
                      doc_id=doc.id)
        response = self.get(url)
        self.assert_200(response)

        url = url_for("documents.document_download",
                      community_id=self.community.slug,
                      doc_id=doc.id)
        response = self.get(url)
        self.assert_200(response)
        assert response.headers['Content-Type'] == content_type

        content = self.open_file(title).read()
        assert response.data == content

        if test_preview:
            url = url_for("documents.document_preview_image",
                          community_id=self.community.slug,
                          doc_id=doc.id,
                          size=500)
            response = self.get(url)
            if assert_preview_available:
                self.assert_200(response)
                assert response.headers['Content-Type'] == 'image/jpeg'
            else:
                # redirect to 'missing image'
                self.assert_302(response)
                assert response.headers['Cache-Control'] == 'no-cache'

        url = url_for("documents.document_delete",
                      community_id=self.community.slug,
                      doc_id=doc.id)
        response = self.client.post(url)
        self.assert_302(response)

        url = url_for("documents.document_view",
                      community_id=self.community.slug,
                      doc_id=doc.id)
        response = self.get(url)
        self.assert_404(response)

    def test_text_upload(self):
        NAME = 'wikipedia-fr.txt'
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(NAME, "text/plain", test_preview=False)

    def test_pdf_upload(self):
        NAME = u"onepage.pdf"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(NAME, "application/pdf")

    def test_image_upload(self):
        NAME = u"picture.jpg"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(NAME, "image/jpeg")

    def test_binary_upload(self):
        NAME = u"random.bin"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(NAME,
                              "application/octet-stream",
                              assert_preview_available=False)

    def test_explore_archive(self):
        from abilian.sbe.apps.documents.views.folders import explore_archive
        fd = self.open_file('content.zip')
        result = [(u'/'.join(path), f)
                  for path, f in explore_archive(fd, u'content.zip')]
        assert result == [(u'', fd)]

        fd = self.open_file('content.zip')
        archive_content = explore_archive(fd, u'content.zip', uncompress=True)
        result = {u'/'.join(path) + u'/' + f.filename
                  for path, f in archive_content}
        assert result == {
            u'existing-doc/file.txt',
            u'existing-doc/subfolder_in_renamed/doc.txt', u'folder 1/doc.txt',
            u'folder 1/dos cp437: é.txt', u'folder 1/osx: utf-8: é.txt'
        }

    def test_zip_upload_uncompress(self):
        folder = Folder(title=u'folder 1', parent=self.community.folder)
        self.session.add(folder)
        self.session.flush()
        folder = self.community.folder
        files = []
        files.append((StringIO('A document'), u'existing-doc', 'text/plain'))
        files.append((self.open_file('content.zip'), u'content.zip',
                      'application/zip'))
        data = {'file': files, 'action': 'upload', 'uncompress_files': True}
        url = url_for("documents.folder_post",
                      community_id=self.community.slug,
                      folder_id=folder.id)
        with self.client_login(self.user.email, password='azerty'):
            response = self.client.post(url, data=data)

        expected = {u'existing-doc', u'folder 1', u'existing-doc-1'}
        self.assert_status(response, 302)
        assert expected == {f.title for f in folder.children}
        expected = {u'folder 1', u'existing-doc-1'}
        assert expected == {f.title for f in folder.subfolders}

    def test_zip(self):
        with self.client_login(self.user.email, password='azerty'):
            title = u"onepage.pdf"
            content_type = "application/pdf"
            data = {
                'file': (self.open_file(title), title, content_type),
                'action': 'upload',
            }
            folder = self.community.folder
            url = url_for("documents.folder_post",
                          community_id=self.community.slug,
                          folder_id=folder.id)
            response = self.client.post(url, data=data)
            self.assert_302(response)

            doc = folder.children[0]
            data = {'action': 'download',
                    'object-selected': ["cmis:document:%d" % doc.id]}
            url = url_for("documents.folder_post",
                          community_id=self.community.slug,
                          folder_id=folder.id)
            response = self.client.post(url, data=data)
            self.assert_200(response)
            assert response.content_type == 'application/zip'

            zipfile = ZipFile(StringIO(response.data))
            assert zipfile.namelist() == [title]

    def test_recursive_zip(self):
        with self.client_login(self.user.email, password='azerty'):
            data = {'action': 'new', 'title': u'my folder'}
            folder = self.community.folder
            url = url_for("documents.folder_post",
                          community_id=self.community.slug,
                          folder_id=folder.id)
            response = self.client.post(url, data=data)
            self.assert_302(response)

            my_folder = folder.children[0]

            title = u"onepage.pdf"
            content_type = "application/pdf"
            data = {
                'file': (self.open_file(title), title, content_type),
                'action': 'upload',
            }
            url = url_for("documents.folder_post",
                          community_id=self.community.slug,
                          folder_id=my_folder.id)
            response = self.client.post(url, data=data)
            self.assert_302(response)

            data = {'action': 'download',
                    'object-selected': ["cmis:folder:%d" % my_folder.id]}
            url = url_for("documents.folder_post",
                          community_id=self.community.slug,
                          folder_id=folder.id)
            response = self.client.post(url, data=data)
            self.assert_200(response)
            assert response.content_type == 'application/zip'

            zipfile = ZipFile(StringIO(response.data))
            assert zipfile.namelist() == ['my folder/' + title]

    def test_document_send_by_mail(self):
        mail = self.app.extensions['mail']
        folder = self.community.folder

        attachment = 'Content-Disposition: attachment; filename="{}"'.format
        attachment_utf8 = 'Content-Disposition: attachment;\n filename*="UTF8\'\'{}"'.format

        with self.client_login(self.user.email, password='azerty'):
            # upload files
            for filename in (u'ascii title.txt', u'utf-8 est arrivé!.txt'):
                content_type = "text/plain"
                data = {
                    'file': (StringIO('file content'), filename, content_type),
                    'action': 'upload',
                }
                url = url_for("documents.folder_post",
                              community_id=self.community.slug,
                              folder_id=folder.id)
                self.client.post(url, data=data)

            ascii_doc = folder.children[0]
            unicode_doc = folder.children[1]

            def get_send_url(doc_id):
                return url_for('documents.document_send',
                               community_id=self.community.slug,
                               doc_id=doc_id)

            # mail ascii filename
            with mail.record_messages() as outbox:
                url = get_send_url(ascii_doc.id)
                rv = self.client.post(url,
                                      data={'recipient': 'dest@example.com',
                                            'message': u'Voilà un fichier'})
                self.assertEqual(rv.status_code, 302,
                                 "expected 302, got:" + rv.status)
                assert len(outbox) == 1
                msg = outbox[0]
                self.assertEqual(msg.subject,
                                 u'[Abilian Test] Unknown sent you a file')
                assert msg.recipients == [u'dest@example.com']
                expected_disposition = attachment('ascii title.txt')
                msg = str(msg)
                assert expected_disposition in msg

            # mail unicode filename
            with mail.record_messages() as outbox:
                url = get_send_url(unicode_doc.id)
                rv = self.client.post(url,
                                      data={'recipient': 'dest@example.com',
                                            'message': u'Voilà un fichier'})
                assert rv.status_code == 302
                assert len(outbox) == 1
                msg = outbox[0]
                self.assertEqual(msg.subject,
                                 u'[Abilian Test] Unknown sent you a file')
                self.assertEqual(msg.recipients, [u'dest@example.com'])
                expected_disposition = attachment_utf8(
                    'utf-8%20est%20arriv%C3%A9%21.txt')
                msg = str(msg)
                self.assertTrue(expected_disposition in msg,
                                (expected_disposition, msg))


class TestPathIndexable(unittest.TestCase):

    class MockPath(PathAndSecurityIndexable):

        def __init__(self, id, parent=None):
            self.id = id
            self.parent = parent

    def setUp(self):
        id_gen = count()
        obj = self.MockPath(id_gen.next())
        obj = self.MockPath(id_gen.next(), parent=obj)
        obj = self.MockPath(id_gen.next(), parent=obj)
        self.obj = self.MockPath(id_gen.next(), parent=obj)

    def test_iter_to_root(self):
        assert [o.id for o in self.obj._iter_to_root()] == [3, 2, 1, 0]
        assert [o.id
                for o in self.obj._iter_to_root(skip_self=True)] == [2, 1, 0]

    def test_indexable_parent_ids(self):
        self.assertEqual(self.obj._indexable_parent_ids, u'/0/1/2')



# coding=utf-8
"""
"""
from __future__ import absolute_import, unicode_literals

from sqlalchemy import Column, DateTime, Unicode
from sqlalchemy.orm import backref, relationship

from abilian.core.entities import SEARCHABLE, Entity
from abilian.sbe.apps.communities.models import (Community, CommunityIdColumn,
                                                 community_content)


@community_content
class Event(Entity):
    __tablename__ = 'sbe_event'

    community_id = CommunityIdColumn()
    #: The community this event belongs to
    community = relationship(
        Community,
        primaryjoin=(community_id == Community.id),
        backref=backref('events', cascade="all, delete-orphan"))

    description = Column(Unicode, nullable=False, default="", info=SEARCHABLE)

    location = Column(Unicode, nullable=False, default="", info=SEARCHABLE)

    start = Column(DateTime, nullable=False)
    end = Column(DateTime)



# coding=utf-8
"""
"""
from __future__ import absolute_import

import logging

import whoosh.fields as wf
import whoosh.query as wq
from flask import g
from flask_login import current_user

from .models import Membership

logger = logging.getLogger(__name__)

_COMMUNITY_CONTENT_FIELDNAME = 'is_community_content'
_COMMUNITY_CONTENT_FIELD = wf.BOOLEAN()

_COMMUNITY_ID_FIELD = wf.NUMERIC(numtype=int,
                                 bits=64,
                                 signed=False,
                                 stored=True,
                                 unique=False)
_COMMUNITY_SLUG_FIELD = wf.ID(stored=True)

_FIELDS = [
    (_COMMUNITY_CONTENT_FIELDNAME, _COMMUNITY_CONTENT_FIELD),
    ('community_id', _COMMUNITY_ID_FIELD),
    ('community_slug', _COMMUNITY_SLUG_FIELD),
]


def init_app(app):
    """Add community fields to indexing service schema."""
    indexing = app.services['indexing']
    indexing.register_search_filter(filter_user_communities)
    indexing.register_value_provider(mark_non_community_content)

    for name, schema in indexing.schemas.items():
        for fieldname, field in _FIELDS:
            if fieldname in schema:
                if schema[fieldname] is not field:
                    logger.warning(
                        'Field "%s" already in schema %r, replacing with '
                        'expected fieldtype instance', fieldname, schema)
                    del schema._fields[fieldname]
                else:
                    continue

            schema.add(fieldname, field)


def filter_user_communities():
    if g.is_manager:
        return None

    filter_q = wq.Term(_COMMUNITY_CONTENT_FIELDNAME, False)

    if not current_user.is_anonymous:
        ids = Membership.query \
            .filter(Membership.user == current_user) \
            .order_by(Membership.community_id.asc()) \
            .values(Membership.community_id)
        communities = [wq.Term('community_id', i[0]) for i in ids]

        if communities:
            communities = wq.And([wq.Term(_COMMUNITY_CONTENT_FIELDNAME, True),
                                  wq.Or(communities)])
            filter_q = wq.Or([filter_q, communities])

    return filter_q


def mark_non_community_content(document, obj):
    if _COMMUNITY_CONTENT_FIELDNAME not in document:
        document[_COMMUNITY_CONTENT_FIELDNAME] = \
            getattr(obj, _COMMUNITY_CONTENT_FIELDNAME, False)

    return document



# coding=utf-8
"""
"""
from __future__ import absolute_import, unicode_literals

from datetime import datetime
from unittest import TestCase

from flask import url_for
from pytest import skip

from abilian.sbe.apps.communities.tests.base import (CommunityBaseTestCase,
                                                     CommunityIndexingTestCase)

from ..models import Event


class Test(TestCase):

    def test_create_event(self):
        start = datetime.now()
        event = Event(name="Test thread", start=start)
        assert event  # TODO


class IndexingTestCase(CommunityIndexingTestCase):

    @skip
    def test_event_indexed(self):
        start = datetime.now()
        event1 = Event(name="Test event", community=self.community, start=start)
        event2 = Event(name="Test other event", community=self.c2, start=start)
        self.session.add(event1)
        self.session.add(event2)
        self.session.commit()

        svc = self.svc
        obj_types = (Event.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search('event', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search('event', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == event1.object_key

        with self.login(self.user_c2):
            res = svc.search(u'event', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == event2.object_key


class NoLoginViewTest(CommunityBaseTestCase):
    """
    Test correct url response, without login or security involved
    """

    @skip
    def test(self):
        response = self.client.get(url_for("calendar.index",
                                           community_id=self.community.slug))
        self.assert200(response)

# class ViewTestCase(CommunityBaseTestCase):
#     no_login = False
#     SERVICES = ('security',)
#
#     def test_posts_ordering(self):
#         thread = Thread(community=self.community, title=u'test ordering')
#         self.session.add(thread)
#         t1 = datetime(2014, 6, 20, 15, 0, 0)
#         p1 = Post(thread=thread, body_html=u'post 1', created_at=t1)
#         t2 = datetime(2014, 6, 20, 15, 1, 0)
#         p2 = Post(thread=thread, body_html=u'post 2', created_at=t2)
#         self.session.flush()
#         p1_id, p2_id = p1.id, p2.id
#         assert [p.id for p in thread.posts] == [p1_id, p2_id]
#
#         # set post1 created after post2
#         t1 = t1 + timedelta(minutes=2)
#         p1.created_at = t1
#         self.session.flush()
#         self.session.expire(thread)  # force thread.posts refreshed from DB
#         assert [p.id for p in thread.posts] == [p2_id, p1_id]
#
#     def test_create_thread_and_post(self):
#         # activate email reply
#         self.app.config['SBE_FORUM_REPLY_BY_MAIL'] = True
#         self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'
#
#         # create a new user, add him/her to the current community as a MANAGER
#         self.user = User(email=u'user_1@example.com',
#                          password='azerty',
#                          can_login=True)
#         self.session.add(self.user)
#         self.community.set_membership(self.user, MANAGER)
#         self.session.commit()
#         self.client_login(self.user.email, self.user.password)
#
#         mail = self.app.extensions['mail']
#         with mail.record_messages() as outbox:
#             title = u"Brand new thread"
#             content = u"shiny thread message"
#             url = url_for("forum.new_thread", community_id=self.community.slug)
#             data = dict(title=title, message=content)
#             data['__action'] = u"create"
#             data['send_by_email'] = u"y"
#             response = self.client.post(url, data=data)
#             self.assertStatus(response, 302)
#
#             # extract the thread_id from the redirection url in response
#             threadid = response.location.rsplit('/', 2)[1]
#
#             # retrieve the new thread, make sur it has the message
#             url = url_for("forum.thread",
#                           thread_id=threadid,
#                           community_id=self.community.slug,
#                           title=title)
#             response = self.client.get(url)
#             self.assertStatus(response, 200)
#             self.assertIn(content, response.data.decode("utf-8"))
#
#             # check the email was sent with the new thread
#             assert len(outbox) == 1
#             assert outbox[0].subject == u'[My Community] Brand new thread'
#
#         # reset the outbox for checking threadpost email
#         with mail.record_messages() as outbox:
#             content = data['message'] = u"my cherished post"
#             del data['title']
#             response = self.client.post(url, data=data)
#             self.assertStatus(response, 302)
#
#             # retrieve the new thread, make sur it has the message
#             url = url_for("forum.thread",
#                           thread_id=threadid,
#                           community_id=self.community.slug,
#                           title=title)
#             response = self.client.get(url)
#             self.assertStatus(response, 200)
#             self.assertIn(content, response.data.decode("utf-8"))
#
#             # check the email was sent with the new threadpost
#             assert len(outbox) == 1
#             assert unicode(outbox[
#                 0].subject) == u'[My Community] Brand new thread'
#
#     def test_create_thread_informative(self):
#         """
#     Test with 'informative' community. No mail sent, unless user is MANAGER
#     """
#         assert self.community.type == 'informative'
#         # create a new user, add him/her to the current community
#         self.user = User(email=u'user_1@example.com',
#                          password='azerty',
#                          can_login=True)
#         self.session.add(self.user)
#         self.community.set_membership(self.user, MEMBER)
#         self.session.commit()
#
#         title = u"Brand new thread"
#         content = u"shiny thread message"
#         url = url_for("forum.new_thread", community_id=self.community.slug)
#         data = dict(title=title, message=content)
#         data['__action'] = u"create"
#
#         mail = self.app.extensions['mail']
#         self.client_login(self.user.email, self.user.password)
#
#         with mail.record_messages() as outbox:
#             data['send_by_email'] = u"y"  # actually should not be in html form
#             response = self.client.post(url, data=data)
#             self.assertStatus(response, 302)
#             assert len(outbox) == 0
#
#         self.community.set_membership(self.user, MANAGER)
#         self.session.commit()
#
#         with mail.record_messages() as outbox:
#             data['send_by_email'] = u"y"  # should be in html form
#             response = self.client.post(url, data=data)
#             self.assertStatus(response, 302)
#             assert len(outbox) == 1
#
#         with mail.record_messages() as outbox:
#             del data['send_by_email']
#             response = self.client.post(url, data=data)
#             self.assertStatus(response, 302)
#             assert len(outbox) == 0
#
#         self.client_logout()
#
#
# def get_string_from_file(filename='notification.email'):
#     """Load a test email return as string."""
#     filepath = Path(__file__).parent / 'data' / filename
#     with filepath.open('rt', encoding='utf-8') as email_file:
#         email_string = email_file.read()
#     return email_string
#
#
# def get_emailmessage_from_file(filename='notification.email'):
#     """Load a mail parse it into a email.message."""
#     email_string = get_string_from_file(filename)
#     parser = FeedParser()
#     parser.feed(email_string)
#     email_message = parser.close()
#     return email_message
#
#
# class CommandsTest(TestCase):
#
#     @patch('fileinput.input')
#     @patch('abilian.sbe.apps.forum.commands.process_email')
#     def test_parse_forum_email(self, mock_process_email, mock_email):
#         """
#         No processing is tested only parsing into a email.message
#         and verifying inject_email() logic
#         """
#         # first load a test email returned by the mock_email
#         mock_email.return_value = get_string_from_file('notification.email')
#
#         # test the parsing function
#         inject_email()
#
#         # assert the email is read
#         assert mock_email.called
#         # assert a call on the celery task was made implying a message creation
#         assert mock_process_email.delay.called
#
#         ##
#         mock_email.reset_mock()
#         mock_process_email.delay.reset_mock()
#         assert not mock_email.called
#         assert not mock_process_email.delay.called
#
#         mock_email.return_value = get_string_from_file('defects.email')
#         inject_email()
#         assert mock_email.called
#         assert not mock_process_email.delay.called
#
#
# class TasksTest(BaseTestCase):
#
#     def test_build_reply_email_address(self):
#         expected_reply_address = 'test+P-fr-3-4-LJXUDQHH3ELLN4KZSLLSA7SHWI@testcase.app.tld'
#         self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'
#         post = Mock()
#         post.id = 2
#         post.thread_id = 3
#         member = Mock()
#         member.id = 4
#         with self.app.test_request_context(
#                 '/build_reply_email_address',
#                 headers=[('Accept-Language', u'fr')]):
#             replyto = build_reply_email_address('test', post, member,
#                                                 'testcase.app.tld')
#         self.assertIn(expected_reply_address, replyto)
#
#     def test_extract_mail_destination(self):
#         self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'
#         self.app.config['MAIL_SENDER'] = u'test@testcase.app.tld'
#         test_address = 'test+test+P-fr-3-4-LJXUDQHH3ELLN4KZSLLSA7SHWI@testcase.app.tld'
#         infos = extract_email_destination(test_address)
#         self.assertIn(u'fr', infos[0])  # locale
#         self.assertIn(u'3', infos[1])  # thread_id
#         self.assertIn(u'4', infos[2])  # post.id
